<?php

use Mds\Collivery;
use Mds\MdsAddress\MdsAddress;
use Mds\MdsColliveryService;

class ModelShippingMds extends Model {

	public function __construct($registry)
	{

		parent::__construct($registry);

		include_once ('./system/helper/mds/Collivery.php');
		include_once ('./system/helper/mds/MdsService.php');
		include_once ('./system/helper/mds/Collivery.php');
		include_once ('./system/helper/mds/Cache.php');
		include_once ('./system/helper/mds/mds_address.php');

		$user_name = $this->db->query("SELECT `value` FROM " . DB_PREFIX . "setting WHERE `key` = \"mds_username\" ");
		$user_pass = $this->db->query("SELECT `value` FROM " . DB_PREFIX . "setting WHERE `key` = \"mds_password\" ");


		$settings = array(
			'app_name'      => "MDS Shipping Extension Open Cart ". VERSION,
			'app_url'       => HTTP_SERVER,
			'app_version'   => "1.0",
			'user_email'    => $user_name->row['value'],
			'user_password' => $user_pass->row['value']
		);

		$this->collivery = new Collivery($settings);

		$this->mdsColliveryService = new MdsColliveryService($settings);

		$this->mdsAddress = new MdsAddress($settings);

		$this->load->model('setting/setting');

		$this->load->language('shipping/mds');

		$data['mdsErrors'] = array();

	}

	function getQuote($address)
	{

		$customer_query = $this->db->query("SELECT `customer_id` FROM " . DB_PREFIX . "address WHERE address_id = '" . (int)$address['address_id'] . "'");

		$custId = $customer_query->row['customer_id'];

		$customer_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "customer WHERE customer_id = '" . (int)$custId . "'");

		$arrayCustoms = $address['custom_field'];

		foreach($arrayCustoms as $key =>  $arrayCustom)
		{
			$custom_field_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "custom_field_description WHERE custom_field_id = '" . (int)$key . "'");

				if ($custom_field_query->row['name'] == "Location Type") {

					$address['location_type'] = $address['custom_field'][ $key ];

				} elseif ($custom_field_query->row['name'] == "Town") {

					$address['town_id'] = $address['custom_field'][ $key ];

				} elseif ($custom_field_query->row['name'] == "Suburb") {

					$address['suburb_id'] = $address['custom_field'][ $key ];
				}
			}

		$address['email'] = $customer_query->row['email'];
		$address['customer_id'] = $customer_query->row['customer_id'];
		$address['phone'] = $customer_query->row['telephone'];
		$address['custom_field'] = $customer_query->row['custom_field'];

		$parcel = $this->cart->getProducts();



		$orderParams = $this->buildColliveryControlDataArray($address, $parcel);

		if ($this->config->get('mds_insurance')) {

			$orderParams['cover'] = 1;
		} else {
			$orderParams['cover'] = 0;
		}

		$services = $this->collivery->getServices();

		foreach ($services as $key => $service ) {

			$orderParams['service'] = $key;

			if ($orderParams['service'] == 5) {

				$orderParams['collection_time'] = time()+(48*60*60);
			} elseif ($orderParams['service'] == 3) {

				$orderParams['collection_time'] = time()+(72*60*60);
			}

			$serviceDisplay = $this->config->get('mds_serviceDisplayName'.$key);

			$serviceMarkup = $this->config->get('mds_serviceSurcharge'.$key);



			$price = $this->getShippingCost($orderParams);

			$displayPrice = $price['price']['inc_vat'] + (($price['price']['inc_vat'] /100) * $serviceMarkup);

			$displayPrice = round($displayPrice, 2);

				$quote_data[$key] = array(
						'code'         => 'mds.' . $key,
						'title'        => $serviceDisplay,
						'cost'         => $displayPrice,
						'tax_class_id' => 0,
						'text'         =>  "<span style=\"font-weight:bold\">Fee: R".$displayPrice . " inc_vat.</span><br>Expected delivery date: " . date("Y-m-d" ,$price['delivery_time']) . "<br>Expected delivery time: " . date("H:i:s" ,$price['delivery_time']),
				);

		}

		$error = '';

		if ($quote_data) {

			$method_data = array(
					'code'       => 'mds'.$key,
					'title'     => "MDS Collivery",
					'quote'     => $quote_data,
					'sort_order' => $key,
					'error' => $error,
			)  ;
			return $method_data;
		} else {

			Echo "Fail";

			return false;
		}

	}

	/**
	 * @param $params
	 *
	 * @return mixed
	 */
	public function buildColliveryControlDataArray($address, $parcel)
	{
		$colliveryAddressTo = $this->addControlColliveryAddressTo($address);

		$cart = $parcel;

		$colliveryFrom = $this->mdsColliveryService->returnDefaultAddress();



		$colliveryFrom['contacts'] = array_pop($colliveryFrom['contacts']);

		$name = explode(" ",$colliveryFrom['contacts']['full_name']);

		$firstName = $name[0];
		$lastName = $name [1];

		$default_customer = $this->db->query("SELECT * FROM " . DB_PREFIX . "address where address_1 = '".$colliveryFrom['address']['street']."'");



		if ($default_customer->num_rows == 0) {

			$this->db->query("INSERT INTO " . DB_PREFIX . "customer SET firstname = '" . $firstName . "',status = '1',approved = '1',password = '" . $firstName . "', lastname = '" . $lastName . "', email = '" . $this->db->escape($colliveryFrom['contacts']['email']) . "', telephone = '" . $this->db->escape($colliveryFrom['contacts']['cellphone']) . "'");



			$customerId = $this->db->query("SELECT * FROM " . DB_PREFIX . "customer where firstname = '" . $firstName . "' and lastname = '" . $lastName."'");
			$customFields = $this->db->query("SELECT * FROM " . DB_PREFIX . "custom_field_description");

			foreach($customFields->rows as $key => $customField) {

				if ($customFields->rows[$key]['name'] == "Suburb") {
					$data['custom_field'][ $customFields->rows[$key]['custom_field_id'] ] = $colliveryFrom['address']['suburb_id'];

				} elseif ($customFields->rows[$key]['name'] == "Location Type") {
					$data['custom_field'][ $customFields->rows[$key]['custom_field_id'] ] = $colliveryFrom['address']['location_type'];

				} elseif ($customFields->rows[$key]['name'] == "Town") {
					$data['custom_field'][ $customFields->rows[$key]['custom_field_id'] ] = $colliveryFrom['address']['town_id'];

				}

			}


			$this->db->query(
				"INSERT INTO " . DB_PREFIX . "address SET customer_id = '" .  $this->db->escape($customerId->row['customer_id']) . "',firstname = '" . $firstName . "', lastname = '" . $lastName . "',company = '" . $colliveryFrom['address']['company_name'] . "', address_1 = '" . $colliveryFrom['address']['street'] . "', city = '" . $colliveryFrom['address']['town_name'] . "', country_id = '193', zone_id = '2963', custom_field = '" . $this->db->escape(isset($data['custom_field']) ? serialize($data['custom_field']) : '') ."',postcode = '" . $colliveryFrom['address']['zip_code'] . "'"
			);



		}

		$colliveryParams['collivery_to'] = $colliveryAddressTo['address_id'];
		$colliveryParams['contact_to'] = $colliveryAddressTo['contact_id'];
		$colliveryParams['collivery_from'] = $colliveryFrom['address']['address_id'];
		$colliveryParams['contact_from'] = $colliveryFrom['contacts']['contact_id'];
		$colliveryParams['collivery_type'] = '2';

		foreach ($cart as $key => $colliveryProduct) {
			for ($i = 0; $i < $colliveryProduct['quantity']; $i++) {
				$colliveryParams['parcels'][$key] = array(
					'weight' => $colliveryProduct['weight'] / $colliveryProduct['quantity'],
					'height' => $colliveryProduct['height'],
					'width'  => $colliveryProduct['width'],
					'length' => $colliveryProduct['length']
				);
			}
		}

		return $colliveryParams;
	}

	public function addControlColliveryAddressTo($address)
	{

		$addressString = $address['address_1'] . $address['town_id'] . $address['suburb_id'] . $address['postcode'] . $address['firstname'] . " " . $address['lastname'];
		$hash = hash('md5', $addressString);
		$hash = substr($hash, 0, 15);
		$address['custom_field'] = $hash . " | " . $address['address_id'];

		$colliveryParams = $this->setColliveryParamsArray($address);

		$colliveryParams['custom_id'] = $address['custom_field'] ;

		try {

			return $this->mdsColliveryService->addColliveryAddress($colliveryParams);
		} catch (Exception $e) {
			die($e->getMessage());
		}

	}

	/**
	 * @param $params
	 *
	 * @return array
	 */

	function setColliveryParamsArray($addressRow)
	{
		$colliveryParams['company_name'] = $addressRow['company'];
		$colliveryParams['building'] = $addressRow['address_2'];
		$colliveryParams['street'] = $addressRow['address_1'];
		$colliveryParams['location_type'] = $addressRow['location_type'];
		$colliveryParams['suburb'] = $addressRow['suburb_id'];
		$colliveryParams['town'] = $addressRow['town_id'];
		$colliveryParams['zip_code'] = $addressRow['postcode'];
		$colliveryParams['full_name'] = $addressRow['firstname'] . $addressRow['lastname'];
		$colliveryParams['phone'] = $this->customer->getTelephone();
		$colliveryParams['cellphone'] = $this->customer->getTelephone();
		$colliveryParams['email'] = $addressRow['email'];

		return $colliveryParams;
	}

	/**
	 * @param $orderParams
	 *
	 * @return mixed
	 */
	public function getShippingCost($orderParams)
	{


		$validate = $this->mdsColliveryService->validateCollivery($orderParams);

		return $validate;
	}
}


